BlueItemLink = CreateFrame("Frame")
BlueItemLink:SetScript("OnEvent", function(self, event, ...) self[event](self, ...) end)

local _G = _G
local unpack = _G.unpack
local select = _G.select
local gsub = _G.gsub
local format = _G.format
local strmatch = _G.strmatch
local strfind = _G.strfind
local tonumber = _G.tonumber
local GetTime = _G.GetTime
local IsModifiedClick = _G.IsModifiedClick
local IsAltKeyDown = _G.IsAltKeyDown
local IsControlKeyDown = _G.IsControlKeyDown
local IsShiftKeyDown = _G.IsShiftKeyDown
local CreateFrame = _G.CreateFrame
local GetItemIcon = _G.GetItemIcon
local GetScreenWidth = _G.GetScreenWidth
local GetItemInfo = _G.GetItemInfo
local GetSpellLink = _G.GetSpellLink
local GetSpellInfo = _G.GetSpellInfo
local DressUpItemLink = _G.DressUpItemLink
local check, ref, index, item1, item2, side, linkID, linkID2, addID, fs, ws, px, py, icon, StatStain, CowTip, QuestsFuTooltips, LibRC2

local linkTypes = { "item", "enchant", "quest", "spell", "achievement", "talent" }
local linkTypesText = { ["item"] = "아이템ID: ", ["enchant"] = "조제법ID: ", ["quest"] = "퀘스트ID: ", ["spell"] = "주문ID: ", ["achievement"] = "업적ID: ", ["talent"] = "특성ID :" }
local enchantLink = "|cffffd000|H%s|h[%s]|h|r"
local questLink = "|cff808080|H%s|h[%s]|h|r"
local talentLink = "|cff4e96f7|H%s|h[%s]|h|r"
local achievementLink = "|cffffff00|H%s|h[%s]|h|r"
local dummy = function() end
local hooks = {}

function BlueItemLink:OnInit()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	local optf = CreateFrame("Frame", "BlueItemLinkOptionFrame", InterfaceOptionsFramePanelContainer)
	optf:Hide()
	optf.name = "BlueItemLink"
	optf:SetScript("OnShow", function()
		BlueItemLink:CreateOptionFrame()
	end)
	InterfaceOptions_AddCategory(optf)
	SLASH_BLUEITEMLINK1 = "/blueitemlink"
	SLASH_BLUEITEMLINK2 = "/bil"
	SlashCmdList["BLUEITEMLINK"] = function()
		InterfaceOptionsFrame_OpenToCategory(BlueItemLinkOptionFrame)
	end
	LibRC2 = LibStub and LibStub("LibRangeCheck-2.0", true)
	if LibRC2 then
		self:SetScript("OnUpdate", function(self)
			if not(LibRC2 and LibRC2.frame and LibRC2.frame:GetScript("OnUpdate")) then
				LibRC2 = nil
				self:SetScript("OnUpdate", nil)
				self:PLAYER_ENTERING_WORLD()
			end
		end)
	end
end

function BlueItemLink:PLAYER_ENTERING_WORLD()
	if LibRC2 then return end
	BlueItemLink:UnregisterEvent("PLAYER_ENTERING_WORLD")
	self.ref = {}
	self.linked = {}
	ItemRefTooltip.Show = ItemRefTooltip.Hide
	ItemRefTooltip.SetHyperlink = self.SetRefLink
	ItemRefTooltip.SetTrainerService = self.SetTrainerService
	BlueItemLinkDB = (BlueItemLinkDB == nil or (BlueItemLinkDB and BlueItemLinkDB.profiles)) and {} or BlueItemLinkDB
	if not BlueItemLinkDB.scale then
		BlueItemLinkDB.scale = 1.0
		BlueItemLinkDB.escHide = true
		BlueItemLinkDB.showIcon = true
		BlueItemLinkDB.itemID = false
		BlueItemLinkDB.enchantID = false
		BlueItemLinkDB.questID = false
		BlueItemLinkDB.spellID = false
		BlueItemLinkDB.achievementID = false
		BlueItemLinkDB.talentID = false
	end
end

function BlueItemLink:SetScale()
	for i = 1, #self.ref do
		self.ref[i]:SetScale(BlueItemLinkDB.scale)
	end
end

function BlueItemLink:SetIcon()
	if BlueItemLinkDB.showIcon then
		for i = 1, #self.ref do
			self.ref[i].icon:Show()
		end
	else
		for i = 1, #self.ref do
			self.ref[i].icon:Hide()
		end
	end
end

function BlueItemLink:SetEscHide()
	if BlueItemLinkDB.escHide then
		local chk = true
		for i = 1, #UISpecialFrames do
			if UISpecialFrames[i] == "BlueItemLink1" then
				chk = false
				break
			end
		end
		if chk then
			for i = 1, #self.ref do
				tinsert(UISpecialFrames, "BlueItemLink"..i)
			end
		end
	else
		for i = 1, #UISpecialFrames do
			if UISpecialFrames[i]:find("BlueItemLink") then
				tremove(UISpecialFrames, i)
				self:SetEscHide()
				break
			end
		end
	end
end


function BlueItemLink:SetRefLink(link)
	if not link or not BlueItemLink.ref then return end
	ref = nil
	for i = 1, #BlueItemLink.ref do
		if BlueItemLink.ref[i]:IsVisible() then
			if BlueItemLink.ref[i].HyperLink == link then
				BlueItemLink.ref[i].forceUpdate = nil
				BlueItemLink.ref[i]:Hide()
				return
			end
		elseif not ref then
			ref = BlueItemLink.ref[i]
		end
	end
	if not ref then
		index = #BlueItemLink.ref + 1
		ref = CreateFrame("GameTooltip", "BlueItemLink"..index, UIParent, "GameTooltipTemplate")
		if index == 1 then
			BlueItemLink.Hook_SetHyperlink = ref.SetHyperlink
			BlueItemLink:SupportOtherAddOns()
		end
		ref:SetScale(BlueItemLinkDB.scale)
		ref:SetPadding(16)
		ref:SetID(index)
		if BlueItemLinkDB[index] then
			fs = ref:GetEffectiveScale()
			ref:SetPoint("TOPLEFT", UIParent, "TOPLEFT", BlueItemLinkDB[index][1] / fs, BlueItemLinkDB[index][2] / fs)
		else
			ref:SetPoint("CENTER", UIParent, "CENTER")
		end
		ref:EnableMouse(true)
		ref:SetMovable(true)
		ref:RegisterForDrag("LeftButton")
		ref:SetFrameStrata("HIGH")
		ref:SetFrameLevel(GameTooltip:GetFrameLevel() + index * 2)
		ref:SetWidth(128)
		ref:SetHeight(64)
		ref.closeButton = CreateFrame("Button", nil, ref)
		ref.closeButton:SetWidth(32)
		ref.closeButton:SetHeight(32)
		ref.closeButton:SetPoint("TOPRIGHT", ref, "TOPRIGHT", 1, 1)
		ref.closeButton:SetScript("OnClick", BlueItemLink.CloseButtonOnClick)
		ref.closeButton:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
		ref.closeButton:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
		ref.closeButton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD")
		ref.icon = ref:CreateTexture(nil, "OVERLAY")
		ref.icon:SetWidth(40)
		ref.icon:SetHeight(40)
		ref.icon:SetPoint("TOPRIGHT", ref, "TOPLEFT", 1, -3)
		if BlueItemLinkDB.showIcon then
			ref.icon:Show()
		else
			ref.icon:Hide()
		end
		ref:SetScript("OnDragStart", BlueItemLink.OnDragStart)
		ref:SetScript("OnDragStop", BlueItemLink.OnDragStop)
		ref:SetScript("OnTooltipSetItem", GameTooltip:GetScript("OnTooltipSetItem"))
		ref:SetScript("OnHide", BlueItemLink.OnHide)
		ref:SetScript("OnUpdate", BlueItemLink.OnUpdate)
		ref:SetScript("OnEnter", BlueItemLink.OnEnter)
		ref:SetScript("OnLeave", BlueItemLink.OnLeave)
		ref:SetScript("OnMouseDown", BlueItemLink.OnClick)
		ref.SetHyperlink = BlueItemLink.SetHyperlink
		ref.firstText = _G["BlueItemLink"..index.."TextLeft1"]
		GameTooltip_OnLoad(ref)
		BlueItemLink.ref[index] = ref
		if BlueItemLinkDB.escHide then
			tinsert(UISpecialFrames, "BlueItemLink"..index)
		end
	end
	ref.forceUpdate = nil
	ref:SetHyperlink(link)
end

function BlueItemLink:SupportOtherAddOns()
	if IsAddOnLoaded("BlueCraftInfo") then
		ChatFrame_OnHyperlinkShow = function(self, link, text, button)
			if type(self) == "string" then
				SetItemRef(self, link, text)
			else
				SetItemRef(link, text, button)
			end
		end
	end
	if IsAddOnLoaded("StatStain") then
		StatStain = LibStub("AceAddon-3.0"):GetAddon("StatStain")
		if StatStain then
			StatStain.OnTooltipSetItem = function(self, tooltip, ...)
				if self.hooks[tooltip] then
					self.hooks[tooltip].OnTooltipSetItem(tooltip, ...)
				end
				self:Modify(tooltip)
			end
		end
	end
	if IsAddOnLoaded("CowTip") then
		CowTip = LibStub("LibRock-1.0"):GetAddon("CowTip")
		if CowTip then
			CowTip.GameTooltip_OnTooltipSetItemOld = CowTip.GameTooltip_OnTooltipSetItem
			CowTip.GameTooltip_OnTooltipSetItem = function(self, tooltip, ...)
				if self.hooks[tooltip] then
					self.GameTooltip_OnTooltipSetItemOld(self, tooltip, ...)
				end
			end
		end
	end
	if IsAddOnLoaded("beql") then
		beql.GameTooltip_OnTooltipSetItemOld = beql.GameTooltip_OnTooltipSetItem
		beql.GameTooltip_OnTooltipSetItem = function(self, tooltip, ...)
			if self.hooks[tooltip] then
				self.GameTooltip_OnTooltipSetItemOld(self, tooltip, ...)
			end
		end
	end
	if IsAddOnLoaded("FuBar_QuestsFu") and QuestsFu then
		QuestsFuTooltips = QuestsFu:GetModule("Tooltips")
		if QuestsFuTooltips then
			QuestsFuTooltips.OnTooltipSetItemOld = QuestsFuTooltips.OnTooltipSetItem
			QuestsFuTooltips.OnTooltipSetItem = function(self, tooltip, ...)
				if self.hooks[tooltip] then
					QuestsFuTooltips.OnTooltipSetItemOld(self, tooltip, ...)
				end
			end
		end
	end
end

function BlueItemLink:SetHyperlink(link)
	self.icon:SetTexture()
	self.HyperLink = link
	BlueItemLink:SetLinkType(self)
	self:SetOwner(UIParent, "ANCHOR_PRESERVE")
	GameTooltip.SetHyperlink(self, link)
	if self.linkType then
		if linkTypesText[self.linkType] and BlueItemLinkDB[self.linkType.."ID"] then
			self:AddLine(linkTypesText[self.linkType]..self.linkID)
		end
		if self.linkType == "item" then
			self.icon:SetTexture(GetItemIcon(self.HyperLink))
			if IsAddOnLoaded("NuguDrop") then
				NuguDrop_OnShow(ref:GetName())
			end
			if BlueItemLink.linked[self.HyperLink] then
				BlueItemLink.linked[self.HyperLink] = true
				self.updateTimer = GetTime() + 1.5
			end
		elseif self.linkType == "spell" or self.linkType == "enchant" then
			icon = select(3, GetSpellInfo(self.linkID))
			self.icon:SetTexture(icon == "Interface\\Icons\\Temp" and "" or icon)
		elseif self.linkType == "achievement" then
			self.icon:SetTexture(select(10, GetAchievementInfo(self.linkID)))
		end
	end
	self:Show()
end

function BlueItemLink:OnUpdate()
	if self.HyperLink then
		if not self.isMoving and self.updateTimer and self.updateTimer < GetTime() then
			self.updateTimer = nil
			self.forceUpdate = true
			self:SetHyperlink(self.HyperLink)
		end
		if self:GetItem() and not ChatFrameEditBox:IsVisible() then
			if self.isEnter and not self.isCompare and IsModifiedClick("COMPAREITEMS") then
				self.isCompare = true
				BlueItemLink:SetCompareItem(self)
			elseif self.isCompare and (not IsModifiedClick("COMPAREITEMS") or not self.isEnter) then
				self.isCompare = nil
				ShoppingTooltip1:Hide()
				ShoppingTooltip2:Hide()
			end
		end
	end
end

function BlueItemLink:GetItemID(link)
	linkID = strmatch(link, "item:(%d+)")
	if linkID then
		return tonumber(linkID)
	end
	linkID = strmatch(link, "enchant:(%d+)")
	if linkID then
		return -tonumber(linkID)
	end
	return nil
end

function BlueItemLink:OnClick()
	if self.HyperLink and not self.isMoving then
		if BlueItemInfo2 and IsAltKeyDown() and BlueItemLink:GetItemID(self.HyperLink) then
			BlueItemInfo2:AddFavoriteItem(BlueItemLink:GetItemID(self.HyperLink))
		elseif BlueItemInfo2 and IsControlKeyDown() and IsShiftKeyDown() and BlueItemLink:GetItemID(self.HyperLink) then
			BlueItemInfo2:AddMemo(BlueItemLink:GetItemID(self.HyperLink))
		elseif IsModifiedClick("CHATLINK") then
			if self.linkType == "item" then
				if AuctionFrame and AuctionFrame:IsVisible() and BrowseName:IsVisible() then
					BrowseName:SetText(self:GetItem().."")
				else
					ChatFrameEditBox:Show()
					ChatFrameEditBox:Insert(select(2, self:GetItem()))
				end
			elseif self.linkType == "enchant" then
				ChatFrameEditBox:Show()
				ChatFrameEditBox:Insert(format(enchantLink, self.HyperLink, gsub(self.firstText:GetText(), "마법부여: ", "")))
			elseif self.linkType == "quest" then
				ChatFrameEditBox:Show()
				ChatFrameEditBox:Insert(format(questLink, self.HyperLink, self.firstText:GetText()))
			elseif self.linkType == "spell" then
				ChatFrameEditBox:Show()
				ChatFrameEditBox:Insert(GetSpellLink(self.linkID))
			elseif self.linkType == "talent" then
				ChatFrameEditBox:Show()
				ChatFrameEditBox:Insert(format(talentLink, self.HyperLink, self.firstText:GetText()))
			elseif self.linkType == "achievement" then
				ChatFrameEditBox:Show()
				ChatFrameEditBox:Insert(format(achievementLink, self.HyperLink, self.firstText:GetText()))
			else
				ChatFrameEditBox:Show()
				ChatFrameEditBox:Insert(self.HyperLink)
			end
		elseif IsModifiedClick("DRESSUP") and (self.linkType == "item" or self.linkType == "enchant:") then
			DressUpItemLink(self.HyperLink)
		end
	end
end

function BlueItemLink:OnEnter()
	self.isEnter = true
end

function BlueItemLink:OnLeave()
	self.isEnter = nil
end

function BlueItemLink:CloseButtonOnClick()
	self.forceUpdate = nil
	self:GetParent():Hide()
end

function BlueItemLink:OnDragStart()
	self.isMoving = true
	self:StartMoving()
end

function BlueItemLink:OnDragStop()
	self.isMoving = nil
	self:StopMovingOrSizing()
	fs, ws = self:GetEffectiveScale(), UIParent:GetEffectiveScale()
	BlueItemLinkDB[self:GetID()] = { self:GetLeft() * fs, self:GetTop() * fs - UIParent:GetTop() * ws }

end

function BlueItemLink:OnHide()
	if not self.forceUpdate then
		self.HyperLink = nil
	end
	self.isCompare = nil
	self.isEnter = nil
	self.isMoving = nil
	self.updateTimer = nil
	self.linkType = nil
	self.linkID = nil
end

function BlueItemLink:SetCompareItem(tooltip)
	if tooltip.HyperLink == nil then return end
	item1, item2 = nil
	if ( ShoppingTooltip1:SetHyperlinkCompareItem(tooltip.HyperLink, 1) ) then
		item1 = true
	end
	if ( ShoppingTooltip2:SetHyperlinkCompareItem(tooltip.HyperLink, 2) ) then
		item2 = true
	end
	side = ((GetScreenWidth() - tooltip:GetRight()) < tooltip:GetLeft()) and 1 or 2
	if item1 then
		ShoppingTooltip1:SetOwner(tooltip, "ANCHOR_NONE")
		ShoppingTooltip1:ClearAllPoints()
		if side == 1 then
			ShoppingTooltip1:SetPoint("TOPRIGHT", tooltip, "TOPLEFT", 0, -45)
		else
			ShoppingTooltip1:SetPoint("TOPLEFT", tooltip, "TOPRIGHT", 0, -10)
		end
		ShoppingTooltip1:SetHyperlinkCompareItem(tooltip.HyperLink, 1)
		ShoppingTooltip1:Show()
		if item2 then
			ShoppingTooltip2:SetOwner(ShoppingTooltip1, "ANCHOR_NONE")
			ShoppingTooltip2:ClearAllPoints()
			if side == 1 then
				ShoppingTooltip2:SetPoint("TOPRIGHT", "ShoppingTooltip1", "TOPLEFT", 0, 0)
			else
				ShoppingTooltip2:SetPoint("TOPLEFT", "ShoppingTooltip1", "TOPRIGHT", 0, 0)
			end
			ShoppingTooltip2:SetHyperlinkCompareItem(tooltip.HyperLink, 2)
			ShoppingTooltip2:Show()
		end
	end
end

function BlueItemLink:SetLinkType(frame)
	if frame.HyperLink then
		for i = 1, #linkTypes do
			linkID = tonumber(strmatch(frame.HyperLink, linkTypes[i]..":(%d+)"))
			if linkID then
				frame.linkType = linkTypes[i]
				frame.linkID = linkID
				return
			end
		end
	end
	frame.linkType = nil
	frame.linkID = nil
end

BlueItemLink:OnInit()